//////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 G-Truc Creation (www.g-truc.net)
//////////////////////////////////////////////////////////////////////////////////
// Author  : Christophe [Groove] Riccio (contact [at] g-truc [point] net)
// Created : 2005-01-27
// Updated : 2006-01-04
// Licence : This source is under GNU LGPL licence
// File    : _mat3.h
//////////////////////////////////////////////////////////////////////////////////

#ifndef __mat3_h__
#define __mat3_h__

namespace glm
{
    template <typename T> class _xvec2;
    template <typename T> class _xvec3;
    template <typename T> class _xquat;

	//!< \brief Template for 3 * 3 matrix of floating-point numbers. (From GLSL 1.10.59 specification)
	template <typename T> class _xmat3
    {
    private:
        // Data 
        _xvec3<T> value[3];

    public:
        //int _size() const {return 3;}

    public:
        // Constructors
	    _xmat3();
        explicit _xmat3(const T x);
        explicit _xmat3(const T x0, const T y0, const T z0,
                        const T x1, const T y1, const T z1,
                        const T x2, const T y2, const T z2);
	    explicit _xmat3(const _xvec3<T> & v0, 
                        const _xvec3<T> & v1,
                        const _xvec3<T> & v2);
        //explicit _xmat3(const T* a);
        // GL_GTX_euler_angles
        //explicit _xmat3(const _xvec2<T> & angles);
        //explicit _xmat3(const _xvec3<T> & angles);

        // Conversions
        //explicit _xmat3(const glm::_xquat<T> & q);

        // Accesses
        _xvec3<T>& operator[](int i) {return value[i];}
        const _xvec3<T> & operator[](int i) const {return value[i];}
        operator T*() {return &value[0][0];}
	    operator const T*() const {return &value[0][0];}

        // Operators
        _xmat3<T>& operator=(const _xmat3<T>& m);
	    _xmat3<T> operator+ (const _xmat3<T>& m) const;
	    _xmat3<T>& operator+= (const _xmat3<T>& m);
	    _xmat3<T> operator- (const _xmat3<T>& m) const;
	    _xmat3<T>& operator-= (const _xmat3<T>& m);
	    _xmat3<T> operator* (const T s) const;
	    _xmat3<T>& operator*= (const T s);
	    _xvec3<T> operator* (const _xvec3<T>& v) const;
	    _xmat3<T> operator/ (const T s) const;
	    _xmat3<T>& operator/= (const T s);
        _xmat3<T> operator* (const _xmat3<T>& m) const;
        _xmat3<T>& operator*= (const _xmat3<T>& m);
        _xmat3<T> operator/ (const _xmat3<T>& m) const;
	    _xmat3<T>& operator/= (const _xmat3<T>& m);
	    _xmat3<T> operator- () const;
        _xmat3<T> operator++ ();
	    const _xmat3<T> operator++ (int n) const;
	    _xmat3<T> operator-- ();
	    const _xmat3<T> operator-- (int n) const;
    };

    template <typename T> _xmat3<T> operator* (const T s, const _xmat3<T>& m)
    {
        return m * s;
    }

    template <typename T> _xvec3<T> operator* (const _xvec3<T>& v, const _xmat3<T>& m)
    {
        _xmat3<T> Transpose;
        Transpose[0][0] = m[0][0];
        Transpose[1][0] = m[0][1];
        Transpose[2][0] = m[0][2];

        Transpose[0][1] = m[1][0];
        Transpose[1][1] = m[1][1];
        Transpose[2][1] = m[1][2];

        Transpose[0][2] = m[2][0];
        Transpose[1][2] = m[2][1];
        Transpose[2][2] = m[2][2];
        return Transpose * v;
    }
} //namespace glm

#endif //__mat3_h__
