namespace o {
namespace core {

// CComponentCreator
template <typename D>
CComponentCreator<D>::CComponentCreator()
{}

template <typename D>
CComponentCreator<D>::~CComponentCreator()
{}

template <typename D>
IComponent* CComponentCreator<D>::Create(handle ObjectId, IParameter* Parameter)
{
	return new_object(D(ObjectId, Parameter));
}

template <typename D>
bool CComponentCreator<D>::Destroy(IComponent* Component)
{
	if(!Component)
		return false;
	delete_object(Component);
	return true;
}

// CComponentFactory
template<typename K>
CComponentFactory<K>::CComponentFactory()
{}

template<typename K>
CComponentFactory<K>::~CComponentFactory()
{
	for
	(
		TCreatorsIt it = m_Creators.begin();
		it != m_Creators.end();
		it++
	)
	{
		delete it->second;
	}
}

template<typename K>
bool CComponentFactory<K>::Register(K Key, IComponentCreator* pCreator)
{
	if(m_Creators.find(Key) != m_Creators.end())
		return false;
	m_Creators.insert(std::make_pair(Key, pCreator));
	return true;
}

template<typename K>
bool CComponentFactory<K>::UnRegister(K Key)
{
	TCreatorsIt it = m_Creators.find(Key);
	if(it == m_Creators.end())
		return false;
	delete it->second;
	m_Creators.erase(it);
	return true;
}

template<typename K>
IComponent* CComponentFactory<K>::CreateObject(K Key, handle ObjectId, IParameter* Parameter) const
{
	TCreatorsCit it = m_Creators.find(Key);
	if(it == m_Creators.end())
		return 0;
	return it->second->Create(ObjectId, Parameter);
}

template<typename K>
bool CComponentFactory<K>::DestroyObject(IComponent* pObject)
{
	if(!pObject)
		return false;
	delete pObject;
	return true;
}

}//namespace core
}//namespace o
